/*
 *     RefactoringHistory.java  Mar 26, 2004
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

package org.jtool.jrbx.gui;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import java.io.*;

public class RefactoringHistory {
    private static final String HISTORY_FILENAME = ".jrbx.history";
    private static HistoryAllDialog historyAllDialog;
    private static List records = new ArrayList();
    private static LinkedList memories = new LinkedList();
    private static HashMap memoriesMap = new HashMap();

    public static void setHistoryAllDialog(HistoryAllDialog dialog) {
        historyAllDialog = dialog;
    }

    public static void init() {
        if (!load()) {
            // System.out.println("NOT FOUND HISTORY");
        }
        memories.clear();
        memoriesMap.clear();
    }

    public static void memorize(RefactoringUndoManager undoManager) {
        LinkedList sequance = (LinkedList)memoriesMap.get(undoManager);
        if (sequance != null) {
            memories.remove(sequance);
        }

        LinkedList nsequance = undoManager.getRefactoringCommands();
        if (nsequance.size() != 0) {
            memories.add(new LinkedList(nsequance));
            memoriesMap.put(undoManager, nsequance);
        }

        historyAllDialog.update();
    }

    private static boolean load() {
        try {
            InputStream is = new BufferedInputStream(new FileInputStream(getHistoryFileName()));
            ObjectInputStream ois = new ObjectInputStream(is);
            records = (ArrayList)ois.readObject();
            is.close();

        } catch (ClassNotFoundException e) {
            // System.out.println("CLASS NOT FOUND");
            return false;
        } catch (IOException e) {
            // System.out.println("IO EXCEPTION");
            return false;
        }
        return true;
    }        

    public static boolean store() {
        recordAllMomories();

        try {
            OutputStream os = new BufferedOutputStream(new FileOutputStream(getHistoryFileName()));
            ObjectOutputStream oos = new ObjectOutputStream(os);
            oos.writeObject(records);
            oos.flush();
            os.close();

        } catch (IOException e) {
            // System.out.println("IO EXCEPTION");
            return false;
        }
        return true;
    }        

    private static String getHistoryFileName() {
        return SystemProperties.getApplicationDir() + File.separator + HISTORY_FILENAME;
    }

    private static void recordAllMomories() {
        Iterator it = memories.iterator();
        while (it.hasNext()) {
            LinkedList sequence = (LinkedList)it.next();
            records.add(sequence);
        }
    }

    public static List collectAllSequences() {
        List sequences = new ArrayList();
        sequences.addAll(records);
        sequences.addAll(memories);
        return sequences;
    }
}
