/*
 *     RefactoringUndoManager.java  Nov 23, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.gui;
import org.jtool.platform.gui.*;
import javax.swing.undo.*;
import java.util.*;

public class RefactoringUndoManager extends CodeUndoManager {
    protected LinkedList commands = new LinkedList();  // RefactoringRecord
    protected Stack redoCommands = new Stack();        // RefactoringRecord

    public RefactoringUndoManager(MainFrame frame) {
        super(frame);
    }

    public void beginCompoundEdit() {
        compoundEdit = new CompoundEdit();
    }

    public void endCompoundEdit(RefactoringRecord record) {
        compoundEdit.end();
        if (compoundEdit.canUndo()) {
            addEdit(compoundEdit);
            presentationNames.put(compoundEdit, record.getCommand());

            record.setEdit(compoundEdit);
            append(record);
        }

        menuPane.updateUndoState(this);
        menuPane.updateRedoState(this);
    }

    public void append(RefactoringRecord record) {
        commands.add(record);
        if (!redoCommands.empty()) {
            redoCommands.clear();
        }
    }

    public void undo() {
        if (isRefactoringCommand(getUndoPresentationName())) {
            if (!commands.isEmpty()) {
                redoCommands.push(commands.removeLast());
            }
        }
        super.undo();
    }

    public void redo() {
        if (isRefactoringCommand(getRedoPresentationName())) {
            if (!redoCommands.empty()) {
                commands.add(redoCommands.pop());
            }
        }
        super.redo();
    }

    private boolean isRefactoringCommand(String name) {
        if (name != null && name.indexOf(":") != -1) {
            return true;
        }
        return false;
    }

    public RefactoringRecord getLastRefactoringCommand(int index) {
        int len = commands.size();
        if (len > index - 1) {
            return (RefactoringRecord)commands.get(len - index);
        }
        return null;
    }

    public RefactoringRecord getLastRefactoringCommand() {
        if (commands.size() != 0) {
            return (RefactoringRecord)commands.getLast();
        }
        return null;
    }

    public boolean isLastRefactoringCommand(String name) {
        RefactoringRecord record = getLastRefactoringCommand();
        if (record != null) {
            String pname = name.substring(0, name.indexOf(":"));
            String rname = record.getCommand().substring(0, record.getCommand().indexOf(":"));
            if (pname.equals(rname)) {
                return true;
            }
        }
        return false;
    }

    public boolean isLastChangeByRefactoring() {
        return isRefactoringCommand(getUndoPresentationName());
    }

    public void undoRefactoring() {
        while (canUndo() && !isRefactoringCommand(getUndoPresentationName())) {
            undo();
        }
        undo();
    }

    public LinkedList getRefactoringCommands() {
        return new LinkedList(commands);
    }

    public void memorize() {
        RefactoringHistory.memorize(this);
    }

    public String getText() {
        StringBuffer buf = new StringBuffer();

        Iterator it = commands.iterator();
        while (it.hasNext()) {
            RefactoringRecord record = (RefactoringRecord)it.next();
            buf.append(record.getSimpleLog());
            buf.append("\n");
        }

        if (buf.length() == 0) {
            buf.append("---- Empty ----");
        }
        return buf.toString();
    }
}
