/*
 *     ExtractSubclassTransformer.java  Feb 1, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.classes;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class ExtractSubclassTransformer extends RefactoringTransformer {
    private JavaClass jclass;
    private String newName;

    private ExtractSubclassTransformer() {
    }

    public ExtractSubclassTransformer(JavaClass jclass, String newName) {
        this.jclass = jclass;
        this.newName = newName;
    }

    public void changeSrc() {
        insertNewClassDecl();
    }

    public void changeRef(JavaClass rclass) {
        Node node = rclass.getSuperClassNode();
        rename(newName, node);
    }

    private void insertNewClassDecl() {
        Node node = jclass.getElem();

        StringBuffer buf = new StringBuffer();
        buf.append("\n\n");
        buf.append("class " + newName);

        String extendsName = jclass.getName();
        if (extendsName.length() != 0) {
            buf.append(" extends " + jclass.getName());
        }
        buf.append(" {\n");
        buf.append(insertNewConstructor());
        buf.append("}");

        insertCodeAfter(buf.toString(), node);
    }

    public String insertNewConstructor() {
        StringBuffer buf = new StringBuffer();

        Iterator it = jclass.getConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());
            MethodCode mcode = new MethodCode(null, newName);

            Iterator itp = jm.getParameterNodes().iterator();
            while (itp.hasNext()) {
                JavaParameter jp = new JavaParameter((Node)itp.next());
                mcode.addParameter(jp.getType(), jp.getTypeFqn(), jp.getName());
            }

            buf.append("    ");
            buf.append(mcode.getCode());
            buf.append(" {\n");
            buf.append("        ");
            buf.append("super(");
            buf.append(mcode.getParameterNames());
            buf.append(");\n");
            buf.append("    }\n");
        }
        return buf.toString();
    }
}
