/*
 *     RenameClassTransformer.java  Feb 2, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.classes;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class RenameClassTransformer extends RefactoringTransformer {
    private JavaClass jclass;
    private String newName;

    private RenameClassTransformer() {
    }

    public RenameClassTransformer(JavaClass jclass, String newName) {
        this.jclass = jclass;
        this.newName = newName;
    }

    public void changeSrc() {
        rename(newName, jclass);
        changeConstructorDeclName();
    }

    public void changeRef(JavaFile rfile) {
        changeTypeName(rfile, newName);
        String pname = jclass.getJavaFile().getPackageName();
        
        if (!rfile.getPackageName().equals(pname)) {
            boolean hasImported = false;
            Iterator it = rfile.getImportNodes().iterator();
            while (it.hasNext()) {
                JavaImport ji = new JavaImport((Element)it.next());

                if (pname.length() != 0) {
                    if (ji.getName().equals(pname + "." + jclass.getName())) {
                        delete(ji);
                        insertImportCode(pname + "." + newName, rfile);
                    }
                }
            }
        }
    }

    private void changeConstructorDeclName() {
        Iterator it = jclass.getConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());

            Node node = jm.getIdentNode();
            rename(newName, node);
        }
    }

    private void changeTypeName(JavaFile jf, String name) {
        ArrayList types = QueryFile.getJavaTypesUsingTheClass(jf, jclass);
        Iterator it = types.iterator();
        while (it.hasNext()) {
            JavaType jt = (JavaType)it.next();
            
            rename(name, jt);
        }
    }
}
