/*
 *     AccessorDialog.java  Jan 11, 2006
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.jrbx.refactor.query.QueryField;
import javax.swing.*;
import java.awt.event.*;

public class AccessorDialog extends SimpleDialog {
    private JTextField getterField;
    private JTextField setterField;
    private String getterName;
    private String setterName;

    public AccessorDialog(JFrame frame, String title, String name) {
        super(frame, title, "Create/Specify accessors of the field " + name + ":", name);
    }

    protected JPanel createCenterPane() {
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));

        getterField = createInputTextField(box, "Getter name: ");
        getterField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (setterField.getText().compareTo("") == 0) {
                    setterField.requestFocus();
                } else {
                    okButton.requestFocus();
                }
            }
        });

        setterField = createInputTextField(box, "Setter name: ");
        setterField.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (getterField.getText().compareTo("") == 0) {
                    getterField.requestFocus();
                } else {
                    okButton.requestFocus();
                }
            }
        });

        String suffixName = QueryField.getSuffixName(name);
        getterField.setText("get" + suffixName);
        setterField.setText("set" + suffixName);

        getterField.requestFocus();
        return box;
    }

    protected void okAction() {
        getterName = getterField.getText();
        setterName = setterField.getText();

        if (!JavaIdentifier.isValid(getterName) || !JavaIdentifier.isValid(setterName)) {
            JOptionPane.showMessageDialog(frame, "Invalid string: \""
              + getterName + "\" and/or \"" + setterName + "\"");
        } else {
            setVisible(false);
            dispose();
        }
    }

    protected void cancelAction() {
        getterName = null;
        setterName = null;
        setVisible(false);
        dispose();
    }

    public static AccessorDialog show(JFrame frame, String title, String name) {
        AccessorDialog dialog = new AccessorDialog(frame, title, name);
        return dialog;
    }

    public String getSetterName() {
        return setterName;
    }

    public String getGetterName() {
        return getterName;
    }

    public static void main(String args[]) {
        if (args.length == 1) {
            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            AccessorDialog dialog = AccessorDialog.show(mainFrame, "Accessor", args[0]);
            System.out.println("Getter = " + dialog.getGetterName());
            System.out.println("Setter = " + dialog.getSetterName());
            System.exit(0);
        }
        System.out.println("java AccessorDialog name");
    }
}
