/*
 *     ClassTreeDialog.java  Aug 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.util.Options;
import org.jtool.jrbx.refactor.query.QueryProject;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class ClassTreeDialog extends TreeDialog {

    public ClassTreeDialog(JFrame frame, String title, String dir, ArrayList list) {
        super(frame, title, dir, list);
        setSize(400, 400);
        setVisible(true);
    }

    protected JTree createTree(String dir, ArrayList list) {
        return new ClassTree(dir, list);
    }

    public void valueChanged(TreeSelectionEvent evt) {
        TreePath path = evt.getPath();
        jelem = ((ClassTree)tree).getJavaClass(path);
        if (jelem != null) {
            setVisible(false);
            dispose();
        }
    }

    public static ClassTreeDialog show(JFrame frame, String title, String dir, ArrayList list) {
        return new ClassTreeDialog(frame, title, dir, list);
    }

    public JavaClass getJavaClass() {
        return (JavaClass)jelem;
    }

    public static void main(String args[]) {
        if (args.length == 3) {
            String dirName = SystemProperties.getCanonicalPath(args[0]);

            SystemProperties.setApplicationDir(".");
            SystemProperties.setDefaultProjectSetting(dirName);
            Options.parse(args);
            SystemProperties.setSapidDir(Options.getOption("-sapiddir"));

            Repository.init();
            Repository.parse();
            Repository.terminate();

            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            ClassTreeDialog dialog = ClassTreeDialog.show(mainFrame, "Class Selection",
                                                          dirName, QueryProject.getAllJavaClassesInProject().getArrayList());
            JavaClass jclass = dialog.getJavaClass();
            System.out.println("NAME = " + jclass.getName());
            System.exit(0);
        }
        System.out.println("java ClassTreeDialog top_directory -sapiddir sapid_dir");
    }
}

class ClassTree extends JTree {
    private ClassTreeNode rootNode;
    private HashMap packageHash = new HashMap();

    public ClassTree(String rootDir, ArrayList list) {
        super((TreeModel)null);
        this.putClientProperty("JTree.lineStyle", "Angled");

        rootNode = new ClassTreeNode(rootDir);
        populateClasses(list);
        this.setModel(new DefaultTreeModel(rootNode));
    }
    
    public JavaClass getJavaClass(TreePath path) {
        Object obj = path.getLastPathComponent();
        ClassTreeNode node = (ClassTreeNode)obj;
        if (node.isLeaf()) {
            return node.getJavaClass();
        }
        return null;
    }

    public void populateClasses(ArrayList list) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            String name = jc.getJavaFile().getPackageName();
            if (name.length() == 0) {
                name = "[Anonymous]";
            }

            ClassTreeNode pnode = (ClassTreeNode)packageHash.get(name);
            if (pnode == null) {
                pnode = new ClassTreeNode(name);
                rootNode.add(pnode);
                packageHash.put(name, pnode);
            }
            ClassTreeNode node = new ClassTreeNode(jc);
            pnode.add(node);
        }
    }
}

class ClassTreeNode extends DefaultMutableTreeNode {
    private String name;
    private JavaClass jclass;

    public ClassTreeNode(String name) {
        super();
        this.name = name;
        jclass = null;
    }

    public ClassTreeNode(JavaClass jclass) {
        super();
        name = jclass.getName();
        this.jclass = jclass;
    }

    public String getName() {
        return name;
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public boolean isLeaf() {
        return jclass != null;
    }

    public boolean getAllowsChildren() {
        return jclass == null;
    }

    public String toString() {
        return getName();
    }
}
