/*
 *     FileListDialog.java  Aug 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.j2xml.JavaXMLText;
import org.jtool.platform.util.Options;
import javax.swing.*;
import java.awt.*;
import java.util.*;

public class FileListDialog extends SimpleDialog {
    private JList list;
    private HashMap fileMap = new HashMap();
    private boolean canceled = false;

    public FileListDialog(JFrame frame, String mesg, ArrayList files) {
        super(frame, "File list", mesg);

        ArrayList titles = new ArrayList();
        Iterator it = files.iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            titles.add(jf.getName());
            fileMap.put(jf.getName(), jf);
        }

        list.setListData(titles.toArray());
        setVisible(true);
    }

    protected JPanel createCenterPane() {
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));

        list = new JList();
        list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        list.setVisibleRowCount(10);
        list.setFixedCellWidth(600);
        list.setFixedCellHeight(18);
        list.setFont(new Font("Dialog", Font.PLAIN, 14));
        list.setBackground(Color.white);

        JScrollPane scrollPane = new JScrollPane(list);
        box.add(scrollPane);

        list.requestFocus();
        return box;
    }

    protected void createCancelButton(JPanel panel) {
    }

    protected void okAction() {
        setVisible(false);
        dispose();
    }

    public static FileListDialog show(JFrame frame, String mesg, ArrayList files) {
        return new FileListDialog(frame, mesg, files);
    }

    private ArrayList getSelectedFiles() {
        ArrayList files = new ArrayList();
        Object[] selectedValues = list.getSelectedValues(); 
        for (int i = 0; i < selectedValues.length; i++) {
            JavaFile jf = (JavaFile)fileMap.get(selectedValues[i]);
            files.add(jf);
        }
        return files;
    }

    public static boolean alert(JFrame frame, String mesg, ArrayList files) {
        FileListDialog dialog = new FileListDialog(frame, mesg, files);
        return dialog.isCanceled();
    }

    private boolean isCanceled() {
        return canceled;
    }

    public static void main(String args[]) {
        if (args.length == 3) {
            String dirName = SystemProperties.getCanonicalPath(args[0]);

            SystemProperties.setApplicationDir(".");
            SystemProperties.setDefaultProjectSetting(dirName);
            Options.parse(args);
            SystemProperties.setSapidDir(Options.getOption("-sapiddir"));

            Repository.init();
            Repository.parse();
            Repository.terminate();

            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            ArrayList jfiles = new ArrayList();
            Iterator it = Repository.getJavaTexts().iterator();
            while (it.hasNext()) {
                JavaXMLText text = (JavaXMLText)it.next();

                JavaFile jfile = new JavaFile(text.getDocument().getDocumentElement());
                jfiles.add(jfile);
            }

            FileListDialog dialog = FileListDialog.show(mainFrame, "Files to be changed", jfiles);
            ArrayList files = dialog.getSelectedFiles();

            it = files.iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();
                System.out.println("NAME = " + jf.getName());
            }
            System.exit(0);
        }

        System.out.println("java FileListDialog top_directory -sapiddir sapid_dir");
    }
}
