/*
 *     FileTreeDialog.java  Feb 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.util.Options;
import org.jtool.jrbx.refactor.query.QueryProject;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

public class FileTreeDialog extends TreeDialog {

    public FileTreeDialog(JFrame frame, String title, String dir, ArrayList list) {
        super(frame, title, dir, list);
        setSize(400, 400);
        setVisible(true);
    }

    protected JTree createTree(String dir, ArrayList list) {
        return new FileTree(dir, list);
    }

    public void valueChanged(TreeSelectionEvent evt) {
        TreePath path = evt.getPath();
        jelem = ((FileTree)tree).getJavaFile(path);
        if (jelem != null) {
            setVisible(false);
            dispose();
        }
    }

    public static JavaFile show(JFrame frame, String title, String dir, ArrayList list) {
        FileTreeDialog dialog = new FileTreeDialog(frame, title, dir, list);
        return dialog.getJavaFile();
    }

    public JavaFile getJavaFile() {
        return (JavaFile)jelem;
    }

    public static void main(String args[]) {
        if (args.length == 3) {
            String dirName = SystemProperties.getCanonicalPath(args[0]);

            SystemProperties.setApplicationDir(".");
            SystemProperties.setDefaultProjectSetting(dirName);
            Options.parse(args);
            SystemProperties.setSapidDir(Options.getOption("-sapiddir"));

            Repository.init();
            Repository.parse();
            Repository.terminate();

            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            JavaFile jfile = FileTreeDialog.show(mainFrame, "File Selection",
                                                 dirName, QueryProject.getAllJavaFiles());
            System.out.println("NAME = " + jfile.getName());
            System.exit(0);
        }
        System.out.println("java FileTreeDialog top_directory -sapiddir sapid_dir");
    }
}

class FileTree extends JTree {
    private FileTreeNode rootNode;
    private HashMap dirHash = new HashMap();

    public FileTree(String rootDir, ArrayList list) {
        super((TreeModel)null);
        this.putClientProperty("JTree.lineStyle", "Angled");

        rootNode = new FileTreeNode(rootDir, rootDir);
        dirHash.put(rootDir, rootNode);
        populateDirectories(rootNode);
        populateFiles(list);
        this.setModel(new DefaultTreeModel(rootNode));
    }

    public JavaFile getJavaFile(TreePath path) {
        Object obj = path.getLastPathComponent();
        FileTreeNode node = (FileTreeNode)obj;
        if (node.isLeaf()) {
            return node.getJavaFile();
        }
        return null;
    }

    public void populateDirectories(FileTreeNode parent) {
        File directory = new File(parent.getFullName());
        String[] names = directory.list();
        for (int i = 0; i < names.length; i++) {
            File file = new File(parent.getFullName(), names[i]);

            if (file.isDirectory() && !file.getName().equals(SystemProperties.JDB_DIR)) {
                FileTreeNode node = new FileTreeNode(names[i], file.getPath());
                parent.add(node);
                dirHash.put(node.getFullName(), node);
            }
        }
    }

    public void populateFiles(ArrayList list) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            FileTreeNode node = (FileTreeNode)dirHash.get(jf.getDirName());
            if (node != null) {
                node.add(new FileTreeNode(jf));
            }
        }
    }
}

class FileTreeNode extends DefaultMutableTreeNode {
    private String name;
    private String fullName;
    private JavaFile jfile;

    public FileTreeNode(String name, String fullName) {
        super();
        this.name = name;
        this.fullName = fullName;
        jfile = null;
    }

    public FileTreeNode(JavaFile jfile) {
        super();
        name = jfile.getBaseName();
        fullName = jfile.getName();
        this.jfile = jfile;
    }

    public String getName() {
        return name;
    }

    public String getFullName() {
        return fullName;
    }

    public JavaFile getJavaFile() {
        return jfile;
    }

    public boolean isLeaf() {
        return jfile != null;
    }

    public boolean getAllowsChildren() {
        return jfile == null;
    }

    public String toString() {
        return getName();
    }
}
