/*
 *     MoveMemberDialog.java  Jan 11, 2006
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.util.Options;
import org.jtool.jrbx.refactor.query.QueryProject;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class MoveMemberDialog extends SimpleDialog {
    private JTextField srcNameField;
    private JTextField dstNameField;
    private JavaClass jclass;

    public MoveMemberDialog(JFrame frame, String title, String srcName, String dir, ArrayList list) {
        super(frame, title, "Input targe class name:", dir, list, srcName);
    }

    protected JPanel createCenterPane(final String dir, final ArrayList list) {
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));

        srcNameField = createInputTextField(box, "Source name: ", 70);
        srcNameField.setBackground(new Color(220, 220, 220));
        srcNameField.setEditable(false);
        srcNameField.setText(name);

        JButton selButton = new JButton("Class");
        selButton.setBackground(Color.white);
        selButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ClassTreeDialog dialog = ClassTreeDialog.show(frame, "Class Selection", dir, list);
                jclass = dialog.getJavaClass();
                dstNameField.setText(jclass.getFqn());
            }
        });

        dstNameField = createInputTextField(box, "Target name: ", selButton);
        dstNameField.setBackground(Color.white);
        dstNameField.setEditable(false);

        selButton.requestFocus();
        return box;
    }

    protected void okAction() {
        if (jclass != null) {
            setVisible(false);
            dispose();
        }
    }

    protected void cancelAction() {
        jclass = null;
        setVisible(false);
        dispose();
    }

    public static MoveMemberDialog show(JFrame frame, String title, String srcName, String dir, ArrayList list) {
        return new MoveMemberDialog(frame, title, srcName, dir, list);
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public static void main(String args[]) {
        if (args.length == 3) {
            String dirName = SystemProperties.getCanonicalPath(args[0]);

            SystemProperties.setApplicationDir(".");
            SystemProperties.setDefaultProjectSetting(dirName);
            Options.parse(args);
            SystemProperties.setSapidDir(Options.getOption("-sapiddir"));

            Repository.init();
            Repository.parse();
            Repository.terminate();

            JFrame mainFrame = new JFrame();
            mainFrame.setSize(100, 100);
            mainFrame.setVisible(true);

            MoveMemberDialog dialog = MoveMemberDialog.show(mainFrame, "Move", "SourceClass",
                                                            dirName, QueryProject.getAllJavaClassesInProject().getArrayList());
            JavaClass jclass = dialog.getJavaClass();
            if (jclass != null) {
                System.out.println("NAME = " + jclass.getFqn());
            }
            System.exit(0);
        }
        System.out.println("java MoveMemberDialog top_directory -sapiddir sapid_dir");
    }
}
