/*
 *     ReferenceDialog.java  Aug 4, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import javax.swing.*;
import java.awt.*;

public class ReferenceDialog extends SimpleDialog {
    private JComboBox refNameComb;
    private String refName;

    public ReferenceDialog(JFrame frame, String title, String name, String[] items) {
        super(frame, title, "Multiple fields referring to the target class " +
          name + " were found.");
        for (int i = 0; i < items.length; i++) {
            refNameComb.addItem(items[i]);
        }
        setVisible(true);
    }

    protected JPanel createCenterPane() {
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.Y_AXIS));

        refNameComb = createComboBox(box, "Please select one among them: ");
        refNameComb.setBackground(Color.white);
        refNameComb.requestFocus();
        return box;
    }

    private JComboBox createComboBox(JPanel box, String title) {
        JPanel panel = new JPanel();

        JLabel label = new JLabel(title);
        panel.add(label);

        JComboBox comboBox = new JComboBox();
        panel.add(comboBox);

        box.add(panel);
        return comboBox;
    }

    protected void okAction() {
        refName = (String)refNameComb.getSelectedItem();
        setVisible(false);
        dispose();
    }

    protected void cancelAction() {
        refName = null;
        setVisible(false);
        dispose();
    }

    public static ReferenceDialog show(JFrame frame, String title, String name, String[] items) {
        return new ReferenceDialog(frame, title, name, items);
    }

    public String getRefName() {
        return refName;
    }

    public static void main(String args[]) {
        JFrame mainFrame = new JFrame();
        mainFrame.setSize(100, 100);
        mainFrame.setVisible(true);

        String[] items = { "a", "b", "c" };
        String type = "Class";

        ReferenceDialog dialog = ReferenceDialog.show(mainFrame, "Reference Object", type, items);
        String name = dialog.getRefName();
        System.out.println("NAME = " + name);
        System.exit(0);
    }
}
