/*
 *     SimpleDialog.java  Jan 11, 2006
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SimpleDialog extends JDialog {
    protected JFrame frame;
    protected Container contentPane;
    protected JButton okButton;
    protected JButton cancelButton;
    protected String name;

    protected SimpleDialog(JFrame frame, String title, String mesg) {
        super(frame, title, true);
        this.frame = frame;
        createPanes(mesg, null, null);
        setVisible(true);
    }

    protected SimpleDialog(JFrame frame, String title, String mesg, String name) {
        super(frame, title, true);
        this.frame = frame;
        this.name = name;
        createPanes(mesg, null, null);
        setVisible(true);
    }

    protected SimpleDialog(JFrame frame, String title, String mesg, String dir, ArrayList list, String name) {
        super(frame, title, true);
        this.frame = frame;
        this.name = name;
        createPanes(mesg, dir, list);
        setVisible(true);
    }

    protected void createPanes(String mesg, String dir, ArrayList list) {
        contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());

        JPanel topPane = new JPanel();
        MultiLineJLabel label = new MultiLineJLabel(mesg, JLabel.CENTER);
        label.setForeground(Color.black);
        topPane.add(label);
        contentPane.add(topPane, "North");

        JPanel centerPane;
        if (dir != null) {
            centerPane = createCenterPane(dir, list);
        } else {
            centerPane = createCenterPane();
        }
        contentPane.add(centerPane, "Center");

        JPanel bottomPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        createOkButton(bottomPane);
        createCancelButton(bottomPane);
        contentPane.add(bottomPane, "South");

        if (frame != null) {
            Point p = frame.getLocationOnScreen();
            this.setLocation(p.x + frame.getSize().width - this.getSize().width - 100, p.y + 10);
        }
        this.pack();
        this.validate();

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                cancelAction();
            }
        });
    }

    protected JPanel createCenterPane() {
        return new JPanel();
    }

    protected JPanel createCenterPane(String dir, ArrayList list) {
        return new JPanel();
    }

    protected JTextField createTextField(JPanel panel, String title) {
        JLabel label = new JLabel(title);
        panel.add(label);

        JTextField textField = new JTextField(20);
        textField.setEditable(true);
        textField.setFont(new Font("Dialog", Font.PLAIN, 14));
        textField.setBackground(Color.white);
        textField.setText("");
        panel.add(textField);

        return textField;
    }

    protected JTextField createInputTextField(JPanel box, String title) {
        JPanel panel = new JPanel();
        JTextField textField = createTextField(panel, title);

        box.add(panel);
        return textField;
    }

    protected JTextField createInputTextField(JPanel box, String title, JButton button) {
        JPanel panel = new JPanel();
        JTextField textField = createTextField(panel, title);

        panel.add(button);
        box.add(panel);
        return textField;
    }

    protected JTextField createInputTextField(JPanel box, String title, int space) {
        JPanel panel = new JPanel();
        JTextField textField = createTextField(panel, title);
        panel.add(Box.createHorizontalStrut(space));

        box.add(panel);
        return textField;
    }

    protected void createOkButton(JPanel panel) {
        okButton = new JButton("Ok");
        panel.add(okButton);
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                okAction();
            }
        });
    }

    protected void okAction() {
    }

    protected void createCancelButton(JPanel panel) {
        cancelButton = new JButton("Cancel");
        panel.add(cancelButton);
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelAction();
            }
        });
    }

    protected void cancelAction() {
    }
}

class MultiLineJLabel extends JComponent {

    MultiLineJLabel (String text) {
        this(text, JLabel.LEFT);
    }

    MultiLineJLabel (String text, int alignment) {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        StringTokenizer st = new StringTokenizer(text, "\n");
        while (st.hasMoreTokens()) {
            add(new JLabel(st.nextToken(), alignment));
        }
    }
}
