/*
 *     TreeDialog.java  Feb 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.dialog;
import org.jtool.platform.xml.wrapper.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public abstract class TreeDialog extends JDialog implements TreeSelectionListener {
    protected JFrame frame;
    protected Container contentPane;
    protected JTree tree;
    protected JavaElement jelem;

    protected TreeDialog(JFrame frame, String title, String dir, ArrayList list) {
        super(frame, title, true);
        this.frame = frame;
        createPanes(dir, list);
    }

    protected abstract JTree createTree(String dir, ArrayList list);

    public abstract void valueChanged(TreeSelectionEvent evt);

    protected void createPanes(String dir, ArrayList list) {
        contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());

        tree = createTree(dir, list);
        tree.setFont(new Font("Dialog", Font.PLAIN, 14));
        tree.setSize(10, 10);

        TreeSelectionModel selectionModel = tree.getSelectionModel();
        selectionModel.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeSelectionListener(this);

        JScrollPane panel = new JScrollPane();
        panel.setViewportView(tree);
        contentPane.add(panel, "Center");

        JPanel bottomPane = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        createCancelButton(bottomPane);
        contentPane.add(bottomPane, "South");

        Point p = frame.getLocationOnScreen();
        this.pack();
        this.setLocation(p.x + frame.getSize().width - this.getSize().width - 10, p.y + 10);
        this.validate();

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent evt) {
                jelem = null;
                setVisible(false);
                dispose();
            }
        });
    }

    protected void createCancelButton(JPanel panel) {
        JButton cancelButton = new JButton("Cancel");
        panel.add(cancelButton);

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jelem = null;
                setVisible(false);
                dispose();
            }
        });
    }
}
