/*
 *     EncapsulateFieldTransformer.java  Feb 9, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
// import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;

public class EncapsulateFieldTransformer extends RefactoringTransformer {
    private JavaField jfield;
    private MethodCode setter;
    private MethodCode getter;
    private JavaMethod existingSetter;
    private JavaMethod existingGetter;

    private EncapsulateFieldTransformer() {
    }

    public EncapsulateFieldTransformer(JavaField jfield, MethodCode setter, MethodCode getter,
                                       JavaMethod existingSetter, JavaMethod existingGetter) {
        this.jfield = jfield;
        this.setter = setter;
        this.getter = getter;
        this.existingSetter = existingSetter;
        this.existingGetter = existingGetter;
    }

    public void changeSrc() {
        if (!jfield.isPrivate()) {
            changeAccessModifier(jfield, "private");
        }

        Node node =  jfield.getJavaClass().getLastMemeberNode();
        if (existingSetter == null) {
            insertSetter(setter, node);
        }
        if (existingGetter == null) {
            insertGetter(getter, node);
        }            
    }

    private Node insertSetter(MethodCode setter, Node node) {
        StringBuffer buf = new StringBuffer();

        buf.append("\n    ");
        buf.append(setter.getCode());
        buf.append(" {\n");

        buf.append("        ");
        buf.append("this.");
        buf.append(jfield.getName());
        buf.append(" = ");
        buf.append(setter.getParameterName(0));
        buf.append(";\n");
        buf.append("    }\n");

        return insertCodeBefore(buf.toString(), node);
    }

    private Node insertGetter(MethodCode getter, Node node) {
        StringBuffer buf = new StringBuffer();

        buf.append("\n    ");
        buf.append(getter.getCode());
        buf.append(" {\n");

        buf.append("        ");
        buf.append("return ");
        buf.append(jfield.getName());
        buf.append(";\n");
        buf.append("    }\n");

        return insertCodeBefore(buf.toString(), node);
    }
}
