/*
 *     PushDownField.java  Apr 18, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import javax.swing.JFrame;

public class PushDownField extends FieldRefactoring {
    private JavaClass dclass;
    private PushDownFieldTransformer transformer;

    public PushDownField() {
        super();
    }

    public void setUpWithOptions(JFrame f, JavaClass dclass) {
        frame = f;
        this.dclass = dclass;
    }
    
    public void preconditions() throws RefactoringException {
        hasChild();
        checkFieldAccess();
        checkMethodCall();

        if (dclass == null) {
            MoveMemberDialog dialog = new MoveMemberDialog(frame, "Push Down Field: " + jfield.getName(), jclass.getFqn(),
                             SystemProperties.getProjectDir(), QueryClass.getAllChildren(jclass).getArrayList());
            dclass = dialog.getJavaClass();

            if (dclass == null) {
                throw new RefactoringException("");
            }
        }

        checkMovedField();
    }

    public void transform() throws RefactoringException {
        transformer = new PushDownFieldTransformer(jfield);
        transformer.changeSrc();
        unparse(jfile);

        transformer.changeDst(dclass);
        unparse(dclass.getJavaFile());
    }

    public void additionalTransformation() throws RefactoringException {
        if (jfield.isStatic()) {
            Iterator it = QueryProject.getAllJavaFilesUsingTheField(jfield).iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();

                transformer.changeRef(jf, dclass.getName());
                unparse(jf);
            }
        }
    }

    public String getLog() {
        return jfield.getName() + " from " + jclass.getName() + " to " + dclass.getName();
    }

    public String getUndoInformation() {
        if (jfield.isPrivate()) {
            return super.getUndoInformation();
        }
        return "F:" + jclass.getName() + "#" + jfield.getName();
    }

    private void hasChild() throws RefactoringException {
        if (QueryClass.getAllChildren(jclass).size() == 0) {
            throw new RefactoringException("Not found:" +
              " children of the class " + jclass.getName());
        }
    }

    private void checkFieldAccess() throws RefactoringException {
        if (QueryClass.usesTheField(jclass, jfield)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " is used in the source class " + jclass.getName());
        }

        if (QueryField.usesPrivateFieldsInTheClass(jfield, jclass)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " uses other private fields in the source class " + jclass.getName());
        }
    }

    private void checkMethodCall() throws RefactoringException {
        if (QueryField.callsPrivateMethodsInTheClass(jfield, jclass)) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " uses other private fields in the source class " + jclass.getName());
        }
    }

    private void checkMovedField() throws RefactoringException {
        if (QueryClass.hasTheField(dclass, jfield.getName())) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " already exists in the target class " + dclass.getName());
        }

        JavaClassList classes = QueryProject.getJavaClassesUsingTheField(jfield);
        classes.remove(jclass);
        classes.removeAll(dclass.getDescendants());
        if (!jfield.isStatic() && classes.size() != 0) {
            throw new RefactoringException("Not move:" +
              " the field " + jfield.getName() + " is used by non-descendants of the target class" + dclass.getName());
        }
    }
}
