/*
 *     SelfEncapsulateField.java  Aug 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class SelfEncapsulateField extends FieldRefactoring {
    private MethodCode setter = null;
    private MethodCode getter = null;
    private JavaMethod existingSetter;
    private JavaMethod existingGetter;
    private SelfEncapsulateFieldTransformer transformer;
    private EncapsulateFieldTransformer etransformer;

    public SelfEncapsulateField() {
        super();
    }

    public void setUpWithOptions(JFrame f) {
        frame = f;

        JavaField jf = new JavaField(elem);
        int index = 0;
        while (jf.getName().charAt(index) == '_') {
            index++;
        }
        String topChar = jf.getName().substring(index, index + 1);
        String suffixName = topChar.toUpperCase() + jf.getName().substring(index + 1);
        setter = getSetter(jf, "set" + suffixName);
        getter = getGetter(jf, "get" + suffixName);
    }
    
    public void preconditions() throws RefactoringException {
        checkPrivate();
        checkSimpleAssignment();

        determineAccessors();
    }

    public void transform() throws RefactoringException {
        etransformer = new EncapsulateFieldTransformer(jfield, setter, getter, existingSetter, existingGetter);
        etransformer.changeSrc();

        transformer = new SelfEncapsulateFieldTransformer(jfield, setter, getter);
        transformer.changeSrc();
        unparse(jfile);
    }

    public String getLog() {
        return jfield.getName() + " through accessors " + getter.getName() + " and " + setter.getName();
    }

    private void checkPrivate() throws RefactoringException {
        if (!jfield.isPrivate()) {
            throw new RefactoringException("Not self-encapsulate:" + 
              " the field " + jfield.getName() + " is not private.\n" +
             "Do \"Self Encapsulate Field\"!");
        }
    }

    private void checkSimpleAssignment() throws RefactoringException {
        if (!QueryField.isInOnlySimpleAssignments(jfield, jclass)) {
            throw new RefactoringException("Not self-encapsulate:" +
              " the field " + jfield.getName() + " is multi-defined in a statement");
        }
    }

    private void determineAccessors() throws RefactoringException {
        if (setter == null || getter == null) {
            AccessorDialog dialog = new AccessorDialog(frame, "Setter/Getter", jfield.getName());
            setter = getSetter(jfield, dialog.getSetterName());
            getter = getGetter(jfield, dialog.getGetterName());
        }
        if (setter == null || getter == null) {
            throw new RefactoringException("");
        }

        existingSetter = jfield.getJavaClass().getJavaMethod(setter.getSignature());
        existingGetter = jfield.getJavaClass().getJavaMethod(getter.getSignature());
        if (existingGetter != null && !existingGetter.getTypeFqn().equals(jfield.getTypeFqn())) {
            throw new RefactoringException("Not encapsulate: " +
              " the return type of the getter " + existingGetter.getName() + " is different from " + jfield.getTypeFqn());
        }
    }
}
