/*
 *     SelfEncapsulateFieldTransformer.java  Feb 9, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.fields;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;

public class SelfEncapsulateFieldTransformer extends RefactoringTransformer {
    private JavaField jfield;
    private MethodCode setter;
    private MethodCode getter;

    private SelfEncapsulateFieldTransformer() {
    }

    public SelfEncapsulateFieldTransformer(JavaField jfield, MethodCode setter, MethodCode getter) {
        this.jfield = jfield;
        this.setter = setter;
        this.getter = getter;
    }

    public void changeSrc() {
        Iterator it = jfield.getJavaClass().getMethodAndConstructorNodes().iterator();
        while (it.hasNext()) {
            JavaMethod jm = new JavaMethod((Node)it.next());

            if (!setter.equals(jm) && !getter.equals(jm)) {
                encapsulate(jm);
            }
        }
    }

    public void run(JavaFile dfile) {
        Iterator it = dfile.getClassNodes().iterator();
        while (it.hasNext()) {
            JavaClass jc = new JavaClass((Node)it.next());

            Iterator itc = jc.getMethodAndConstructorNodes().iterator();
            while (itc.hasNext()) {
                JavaMethod jm = new JavaMethod((Node)itc.next());
                encapsulate(jm);
            }
        }
    }

    private void encapsulate(JavaMethod jm) {
        Iterator it = jm.getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            if (jv.getName().equals(jfield.getName()) &&
                jv.getRefClass().equals(jfield.getJavaClass().getFqn())) {

                if (jv.isUseVariable()) {
                    replaceVariableWithGetter(jv);
                }

                if (jv.isDefVariable()) {
                    replaceVariableWithSetter(jv);
                }
            }
        }
    }

    private void replaceVariableWithGetter(JavaVariable jv) {
        Node node = jv.getFirstChildNode(jv.getElem(), JXConstants.IdentElem);
        rename(getter.getName() + "()", node);
    }

    private void replaceVariableWithSetter(JavaVariable jv) {
        Node node = jv.getFirstChildNode(jv.getElem(), JXConstants.IdentElem);

        Node jvNode;
        if (eliminateDotPrefix((Element)jv.getDotPrefix(), "this")) {
            jvNode = jv.getElem().getParentNode();

        } else {
            jvNode = jv.getElem();
            Node parent = jvNode.getParentNode();
            while (parent.getNodeName().equals(JXConstants.ExpressionElem) &&
                   ((Element)parent).getAttribute(JXConstants.SortAttr).equals(JXConstants.DotOp)) {
                jvNode = parent;
                parent = parent.getParentNode();
            }
        }
        eliminateOp(jvNode);
        rename(setter.getName() + "(", node);

        JavaExpr je = new JavaExpr(jvNode.getParentNode());
        if (je.isPreOp()) {
            delete(je.getElem().getFirstChild());
            if (je.isPreIncOp()) {
                insertCodeAfter(" + 1)", je.getElem());
            } else { // je.isPreDecOp()
                insertCodeAfter(" - 1)", je.getElem());
            }
        } else {
            insertCodeAfter(")",  je.getElem());
        }
    }

    private void eliminateOp(Node node) {
        node = node.getNextSibling();

        while (node != null && !node.getNodeName().equals(JXConstants.ExpressionElem)) {
            String op = node.getFirstChild().getNodeValue();

            if (op.equals(" ") || op.equals("=")) {
                delete(node);

            } else if (op.endsWith("=")) {
                rename(" " + op.substring(0, op.length() - 1) + " ", node);

            } else if (op.endsWith("++")) {
                rename(" + 1", node);
            } else if (op.endsWith("--")) {
                rename(" - 1", node);
            }
            node = node.getNextSibling();
        }
    }
}
