/*
 *     DeleteMethod.java  Apr 18, 200
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class DeleteMethod extends MethodRefactoring {
    private DeleteMethodTransformer transformer;

    public DeleteMethod() {
        super();
    }

    public void setUpWithOptions(JFrame f) {
        frame = f;
    }

    public void preconditions() throws RefactoringException {
        checkAlreadyUsed();
        checkOverriddenOrOverridingMethod();
    }

    public void transform() throws RefactoringException {
        transformer = new DeleteMethodTransformer(jmethod);
        transformer.changeSrc();
        unparse(jfile);
    }

    public String getLog() {
        return jmethod.getName() + " in " + jclass.getName();
    }

    public String getUndoInformation() {
        if (jmethod.isPrivate()) {
            return super.getUndoInformation();
        }
        return "M:" + jclass.getName() + "#" + jmethod.getSignature();
    }

    private void checkAlreadyUsed() throws RefactoringException {
        Iterator it = QueryFile.getAllJavaClasses(jfile).iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (QueryClass.callsTheMethodExceptSelf(jc, jmethod)) {
                throw new RefactoringException("Not delete:" +
                  " the method " + jmethod.getName() + " is called in the class " + jc.getName());
            }
        }
    }

    private void checkOverriddenOrOverridingMethod() throws RefactoringException {
        if (QueryProject.getAllJavaFilesCallingTheMethodExceptSelf(jmethod).size() != 0) {
            throw new RefactoringException("Not delete:" +
              " the method " + jmethod.getName() + " is called by the class in the project");
        }
        
        if (QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, jclass.getDescendants()).size() != 1) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is overridden by descendant of the class " + jclass.getName());
        }
    }
}
