/*
 *     PullUpMethod.java  Apr 18, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class PullUpMethod extends MethodRefactoring {
    private JavaClass dclass;
    private PullUpMethodTransformer transformer;

    public PullUpMethod() {
        super();
    }

    public void setUpWithOptions(JFrame f) {
        frame = f;
    }

    public void preconditions() throws RefactoringException {
        checkNative();
        checkAbstract();
        checkMethodCall();
        checkFieldAccess();

        dclass = jclass.getSuperClass();
        if (dclass == null) {
            throw new RefactoringException("");
        }

        checkMovedMethod();
    }

    public void transform() throws RefactoringException {
        transformer = new PullUpMethodTransformer(jmethod);
        transformer.changeSrc();
        unparse(jfile);

        transformer.changeDst(dclass);
        unparse(dclass.getJavaFile());
    }

    public void additionalTransformation() throws RefactoringException {
        if (jmethod.isStatic()) {
            ArrayList files = QueryProject.getAllJavaFilesCallingTheMethod(jmethod);
            Iterator it = files.iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();
            
                transformer.changeRef(jf, dclass.getName());
                unparse(jf);
            }
        }
    }

    public String getLog() {
        return jmethod.getName() + " from " + jclass.getName() + " to " + dclass.getName();
    }

    public String getUndoInformation() {
        if (jmethod.isPrivate()) {
            return super.getUndoInformation();
        }
        return "M:" + jclass.getName() + "#" + jmethod.getSignature();
    }

    private void checkNative() throws RefactoringException {
        if (jmethod.isNative()) {
            throw new RefactoringException("Not rename:" +
              " the method " + jmethod.getName() + " is native");
        }
    }

    private void checkAbstract() throws RefactoringException {
        if (jmethod.isAbstract()) {
            throw new RefactoringException("Not move:" +
             " the method " + jmethod.getName() + " is abstract");
        }
    }

    private void checkMethodCall() throws RefactoringException {
        JavaClassList classes = QueryProject.getJavaClassesCallingTheMethod(jmethod);
        classes.removeAll(jclass.getDescendants());
        if (!jmethod.isStatic() && classes.size() != 0) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is called by non-descendants of the source class" + jclass.getName());
        }

        if (QueryMethod.callsMethodsInTheClassExceptSelf(jmethod, jclass)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " calls other methods in the source class " + jclass.getName());
        }
    }

    private void checkFieldAccess() throws RefactoringException {
        if (QueryMethod.usesFieldsInTheClass(jmethod, jclass)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " uses fields in the source class " + jclass.getName());
        }
    }

    private void checkMovedMethod() throws RefactoringException {
        if (!isInProjectDir(dclass.getJavaFile())) {
            throw new RefactoringException("Not found:" +
              " the class " + dclass.getFqn() + " in the project");
        }

        if (QueryClass.hasTheMethod(dclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " already exists in the target class " + dclass.getName());
        }

        JavaMethodList methods = QueryMethod.getJavaMethodsWithTheSameSignature(jmethod, dclass.getAncestors());
        Iterator it = methods.iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            JavaClassList classes = QueryClass.getChildrenCallingTheMethod(dclass, jm);
            classes.remove(jclass);
            if (classes.size() != 0) {
                throw new RefactoringException("Not move: " +
                  " the method " + jmethod.getName() +
                  " will hide a method with the same name in the ancestors of the sourece class " + jclass.getName());
            }
        }
    }
}
