/*
 *     PushDownMethod.java  Apr 18, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.methods;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import javax.swing.JFrame;

public class PushDownMethod extends MethodRefactoring {
    private JavaClass dclass;
    private PushDownMethodTransformer transformer;

    public PushDownMethod() {
        super();
    }

    public void setUpWithOptions(JFrame f, JavaClass dclass) {
        frame = f;
        this.dclass = dclass;
    }

    public void preconditions() throws RefactoringException {
        hasChild();
        checkNative();
        checkAbstract();
        checkMethodCall();
        checkFieldAccess();

        if (dclass == null) {
            MoveMemberDialog dialog = new MoveMemberDialog(frame, "Push Down Method: " + jmethod.getName(), jclass.getFqn(),
                             SystemProperties.getProjectDir(), QueryClass.getAllChildren(jclass).getArrayList());
            dclass = dialog.getJavaClass();

            if (dclass == null) {
                throw new RefactoringException("");
            }
        }

        checkMovedMethod();
    }

    public void transform() throws RefactoringException {
        transformer = new PushDownMethodTransformer(jmethod);
        transformer.changeSrc();
        unparse(jfile);

        transformer.changeDst(dclass);
        unparse(dclass.getJavaFile());
    }

    public void additionalTransformation() throws RefactoringException {
        if (jmethod.isStatic()) {
            ArrayList files = QueryProject.getAllJavaFilesCallingTheMethod(jmethod);
            Iterator it = files.iterator();
            while (it.hasNext()) {
                JavaFile jf = (JavaFile)it.next();
            
                transformer.changeRef(jf, dclass.getName());
                unparse(jf);
            }
        }
    }

    public String getLog() {
        return jmethod.getName() + " from " + jclass.getName() + " to " + dclass.getName();
    }

    public String getUndoInformation() {
        if (jmethod.isPrivate()) {
            return super.getUndoInformation();
        }
        return "M:" + jclass.getName() + "#" + jmethod.getSignature();
    }

    private void hasChild() throws RefactoringException {
        if (QueryClass.getAllChildren(jclass).size() == 0) {
            throw new RefactoringException("Not found:" +
              " children of the class " + jclass.getName());
        }
    }

    private void checkNative() throws RefactoringException {
        if (jmethod.isNative()) {
            throw new RefactoringException("Not rename:" +
              " the method " + jmethod.getName() + " is native");
        }
    }

    private void checkAbstract() throws RefactoringException {
        if (jmethod.isAbstract()) {
            throw new RefactoringException("Not move:" +
             " the method " + jmethod.getName() + " is abstract");
        }
    }

    private void checkMethodCall() throws RefactoringException {
        if (QueryClass.callsTheMethodExceptSelf(jclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is called in the source class " + jclass.getName());
        }

        if (QueryMethod.callsPrivateMethodsInTheClassExceptSelf(jmethod, jclass)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " calls other private methods in the source class " + jclass.getName());
        }
    }

    private void checkFieldAccess() throws RefactoringException {
        if (QueryMethod.usesPrivateFieldsInTheClass(jmethod, jclass)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " uses private fields in the source class " + jclass.getName());
        }
    }

    private void checkMovedMethod() throws RefactoringException {
        if (QueryClass.hasTheMethod(dclass, jmethod)) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " already exists in the target class " + dclass.getName());
        }

        JavaClassList classes = QueryProject.getJavaClassesCallingTheMethod(jmethod);
        classes.remove(jclass);
        classes.removeAll(dclass.getDescendants());
        if (!jmethod.isStatic() && classes.size() != 0) {
            throw new RefactoringException("Not move:" +
              " the method " + jmethod.getName() + " is called by non-descendants of the target class" + dclass.getName());
        }
    }
}
