/*
 *     QueryField.java  Aug 8, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.AccessibleChecker;
import org.jtool.platform.xml.JXConstants;
import java.util.*;
import org.w3c.dom.*;

public class QueryField {

    public static boolean usesOtherFields(JavaField jfield) {
        return jfield.getUseVariables().size() != 0;
    }

    public static boolean callsOtherMethods(JavaField jfield) {
        Iterator it = jfield.getMethodCallNodes().iterator();
        while (it.hasNext()) {
            JavaCall ja = new JavaCall((Node)it.next());

            JavaMethod jm = ja.getCalledJavaMethod();
            if (jm != null && ja.isMethodCall()) {
                return true;
            }
        }
        return false;
    }

    public static boolean usesFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        return getJavaFieldsUsingFieldsInTheClass(jfield, jclass).size() != 0;
    }
    
    public static boolean usesFieldsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (usesFieldsInTheClass(jfield, jc)) {
                return true;
            }
        }
        return false;
    }
    
    public static JavaFieldList getJavaFieldsUsingFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        JavaVariableList uses = jfield.getUseVariables();
        Iterator it = uses.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            JavaField jf = jv.getUsedJavaField();
            if (jf != null && jclass.equals(jf.getJavaClass())) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static boolean callsMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        return getJavaMethodsCallingMethodsInTheClass(jfield, jclass).size() != 0;
    }
    
    public static boolean callsMethodsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (callsMethodsInTheClass(jfield, jc)) {
                return true;
            }
        }
        return false;
    }
    
    public static JavaMethodList getJavaMethodsCallingMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        JavaMethodList methods = new JavaMethodList();

        Iterator it = jfield.getMethodCallNodes().iterator();
        while (it.hasNext()) {
            JavaCall ja = new JavaCall((Node)it.next());

            JavaMethod jm = ja.getCalledJavaMethod();
            if (jm != null && ja.isMethodCall() && jclass.equals(jm.getJavaClass())) {
                methods.add(jm);
            }
        }
        return methods;
    }

    public static boolean usesPrivateFieldsInTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = getJavaFieldsUsingFieldsInTheClass(jfield, jclass).iterator();
        while (it.hasNext()) {
            JavaField jf = (JavaField)it.next();

            if (jf.isPrivate()) {
                return true;
            }
        }
        return false;
    }

    public static boolean callsPrivateMethodsInTheClass(JavaField jfield, JavaClass jclass) {
        Iterator it = getJavaMethodsCallingMethodsInTheClass(jfield, jclass).iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (jm.isPrivate()) {
                return true;
            }
        }
        return false;
    }

    public static JavaFieldList getJavaFieldsUsingFieldsInAncestorsOfTheClass(JavaField jfield, JavaClass jclass) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclass.getAncestors().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            fields.addAll(getJavaFieldsUsingFieldsInTheClass(jfield, jc));
        }

        return fields;
    }

    public static JavaFieldList getJavaFieldsWithTheSameName(JavaField jfield, JavaClassList jclasses) {
        JavaFieldList fields = new JavaFieldList();

        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            JavaField jf = jc.getJavaField(jfield.getName());
            if (jf != null) {
                fields.add(jf);
            }
        }
        return fields;
    }

    public static boolean isInOnlySimpleAssignments(JavaField jfield, JavaClass jclass) {
        Iterator it = jclass.getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();

            if (!isInOnlySimpleAssignments(jfield, jm)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isInOnlySimpleAssignments(JavaField jfield, JavaMethod jmethod) {
        Iterator it = jmethod.getDefVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            
            JavaField jf = jv.getUsedJavaField();
            if (jf != null && !isInSimpleAssignment(jv)) {
                return false;
            }
        }
        return true;
    }

    private static boolean isInSimpleAssignment(JavaVariable jv) {
        Node node = jv.getElem().getParentNode();
        if (node != null && node.getNodeName().equals(JXConstants.ExpressionElem)) {
            JavaExpr je = new JavaExpr(node);

            if (je.isInSimpleAssignment()) {
                return true;
            }
        }

        Element elem = (Element)jv.getDotPrefix();
        if (elem != null && elem.getNodeName().equals(JXConstants.ExpressionElem) &&
            elem.getAttribute(JXConstants.SortAttr).equals(JXConstants.ThisRef)) {
            return true;
        }

        return false;
    }

    public static boolean isAccessible(JavaClass jc, JavaField jf) {
        return AccessibleChecker.isAccessible(jc, jf);
    }

    public static JavaMethod guessGettter(JavaField jfield) {
        Iterator it = jfield.getJavaClass().getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            if (checkGetter(jm, jfield)) {
                return jm;
            }
        }
        return null;
    }

    private static boolean checkGetter(JavaMethod jm, JavaField jfield) {
        if (jm == null || jm.getNumParameter() != 0 ||
            !jm.getTypeFqn().equals(jfield.getTypeFqn())) {
            return false;
        }

        ArrayList statements = jm.getStatementNodes();
        if (statements.size() != 1) {  // one is a block and the other is a return statement.
            return false;
        }
        JavaStatement jst = new JavaStatement((Element)statements.get(0));

        JavaVariableList defs = jst.getDefVariables();
        JavaVariableList uses = jst.getUseVariables();
        if (defs.size() != 0 || uses.size() != 1) {
            return false;
        }

        JavaVariable use = (JavaVariable)uses.get(0);
        if (use.isField() && jfield.equals(use.getUsedJavaField())) {
            return true;
        }
        return false;
    }

    public static JavaMethod guessSettter(JavaField jfield) {
        Iterator it = jfield.getJavaClass().getAllMethods().iterator();
        while (it.hasNext()) {
            JavaMethod jm = (JavaMethod)it.next();
            if (checkSetter(jm, jfield)) {
                return jm;
            }
        }
        return null;
    }

    private static boolean checkSetter(JavaMethod jm, JavaField jfield) {
        if (jm == null || jm.getNumParameter() != 1) {
            return false;
        }
        JavaParameter jp = new JavaParameter((Element)jm.getParameterNodes().get(0));

        ArrayList statements = jm.getStatementNodes();
        if (statements.size() != 1) {  // one is a block and the other is an assignment statement.
            return false;
        }
        JavaStatement jst = new JavaStatement((Element)statements.get(0));

        JavaVariableList defs = jst.getDefVariables();
        JavaVariableList uses = jst.getUseVariables();
        if (defs.size() != 1 || uses.size() != 1) {
            return false;
        }

        JavaVariable def = (JavaVariable)defs.get(0);
        JavaVariable use = (JavaVariable)uses.get(0);
        if (def.isField() && jfield.equals(def.getUsedJavaField()) &&
            use.isParameter() && jp.equals(new JavaParameter(use.getDeclaration()))) {
            return true;
        }
        return false;
    }

    public static String getSuffixName(String name) {
        int index = 0;
        while (name.charAt(index) == '_') {
            index++;
        }
        String topChar = name.substring(index, index + 1);

        return topChar.toUpperCase() + name.substring(index + 1);
    }
}
