/*
 *     QueryFile.java  Aug 8, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.SystemProperties;
import java.util.*;
import org.w3c.dom.*;

public class QueryFile {

    public static boolean hasTheClass(JavaFile jfile, JavaClass jclass) {
        return hasTheClass(jfile, jclass.getName());
    }

    public static boolean hasTheClass(JavaFile jfile, String className) {
        return jfile.getJavaClass(className) != null;
    }

    public static boolean usesTheType(JavaFile jfile, String fqn) {
        Iterator it = jfile.getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            if (jt.isObjectRef() && jt.getFqn().equals(fqn)) {
                return true;
            }
        }
        return false;
    }

    public static JavaClassList getAllJavaClasses(JavaFile jfile) {
        JavaClassList classes = new JavaClassList();

        Iterator it = jfile.getAllClassesAndInterfaces().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            classes.add(jc);
        }

        return classes;
    }

    public static ArrayList getJavaTypesUsingTheClass(JavaFile jfile, JavaClass jclass) {
        ArrayList types = new ArrayList();

        Iterator it = jfile.getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            if (jt.isObjectRef()) {
                JavaClass jc = jt.getReferredJavaClass();               
                if (jc != null && jc.equals(jclass)) {
                    types.add(jt);
                }
            }
        }
        return types;
    }

    public static boolean usesTheClassExceptSelf(JavaFile jfile, JavaClass jclass) {
        Iterator it = getJavaTypesUsingTheClass(jfile, jclass).iterator();
        while (it.hasNext()) {
            JavaType jt = (JavaType)it.next();
            
            if (!jt.isInTheSameClass(jclass)) {
                return true;
            }
        }
        return false;
    }
    
    public static JavaVariableList getJavaVariablesUsingTheField(JavaFile jfile, JavaField jfield) {
        return QueryProject.getJavaVariablesUsingTheField(jfile, jfield);
    }
      
    public static ArrayList getJavaCallsUsingTheMethod(JavaFile jfile, JavaMethod jmethod) {
        return QueryProject.getJavaCallsUsingTheMethod(jfile, jmethod);
    }

    public boolean containsClassesWithTheSameName(JavaFile jfile, String name) {
        return getJavaClassWithTheSameName(jfile, name) != null;
    }
    
    public static JavaClass getJavaClassWithTheSameName(JavaFile jfile, String name) {
        Iterator it = QueryProject.getAllJavaClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            
            if (jc.isPublic() && jc.getName().equals(name)) {
                return jc;
            }
        }

        it = getAllJavaClassesInTheSamePackage(jfile).iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            
            if ((jc.isProtected() || jc.isDefault()) && jc.getName().equals(name)) {
                return jc;
            }
        }
        return null;
    }
    
    private static JavaClassList getAllJavaClassesInTheSamePackage(JavaFile jfile) {
        JavaClassList classes = new JavaClassList();

        Iterator it = QueryProject.getAllJavaClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (jfile.isInTheSamePackage(jc.getJavaFile())) {
                classes.add(jc);
            }
        }
        return classes;
    }
        
    public static ArrayList getAllFilesInTheSamePackage(JavaFile jfile) {
        ArrayList files = new ArrayList();

        Iterator it = QueryProject.getAllJavaFiles().iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            if (jfile.isInTheSamePackage(jf)) {
                files.add(jf);
            }
        }
        return files;
    }
    
    public static boolean hasImported(JavaFile jfile, JavaClass jclass) {
            Iterator it = jfile.getImportNodes().iterator();
            while (it.hasNext()) {
                JavaImport ji = new JavaImport((Element)it.next());
                
                if (QueryImport.hasImported(ji, jclass)) {
                    return true;
                }
            }
            return false;
        }        
}
