/*
 *     QueryGraph.java  Aug 17, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.graph.pdg.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;

public class QueryGraph {

    public static CFGNode getDeclaraionCFGNode(CFG cfg, GraphCompSet cfgNodes, JavaVariable jv) {
        GraphCompSet candidates = getCandidatesForDeclaration(cfgNodes, jv);

        if (candidates.size() == 0) {
            return null;
        } else if (candidates.size() == 1) {
            return (CFGNode)candidates.getFirst();
        } else {
            return traverseBackward(cfg, candidates);
        }
    }

    private static GraphCompSet getCandidatesForDeclaration(GraphCompSet cfgNodes, JavaVariable jv) {
        GraphCompSet candidates = new GraphCompSet();

        Iterator it = cfgNodes.iterator();
        while (it.hasNext()) {
            CFGNode cfgNode = (CFGNode)it.next();

            if (cfgNode.hasDefVariable() && !cfgNode.isLocalSt()) {
                CFGStatement node = (CFGStatement)cfgNode;
                if (node.containsDefVariable(jv)) {
                    candidates.add(cfgNode);
                }
            }
        }
        return candidates;
    }

    private static CFGNode traverseBackward(CFG cfg, GraphCompSet candidates) {
        GraphCompSet path = cfg.getNodes();
        
        Iterator it = candidates.iterator();
        while (it.hasNext()) {
            CFGNode cfgNode = (CFGNode)it.next();

            GraphCompSet p = cfg.getBackwardReachableNodesWithoutLoopback(cfgNode, cfg.getStartNode());
            path = path.intersection(p);
        }
        return (CFGNode)path.getFirst();
    }

    public static void traverseForwardDD(GraphCompSet nodes, PDGNode pdgNode, JavaVariable jv) {
        nodes.add(pdgNode);

        Iterator it = pdgNode.getOutgoingDDEdges().iterator();
        while (it.hasNext()) {
            DD edge = (DD)it.next();

            if (jv.equals(edge.getVariable())) {
                PDGNode node = (PDGNode)edge.getDstNode();
                if (!nodes.contains(node)) {
                    traverseForwardDD(nodes, node, jv);
                }
            }
        }
    }

    public static void traverseBackwardDD(GraphCompSet nodes, PDGNode pdgNode, JavaVariable jv) {
        nodes.add(pdgNode);

        Iterator it = pdgNode.getIncomingDDEdges().iterator();
        while (it.hasNext()) {
            DD edge = (DD)it.next();

            if (jv.equals(edge.getVariable())) {
                PDGNode node = (PDGNode)edge.getSrcNode();
                if (!nodes.contains(node)) {
                    traverseBackwardDD(nodes, node, jv);
                }
            }
        }
    }

    public static boolean isDefinedAt(CFGNode node, JavaVariable jv) {
        if (node.hasDefVariable()) {
            CFGStatement n = (CFGStatement)node;
            if (n.containsDefVariable(jv)) {
                return true;
            }
        }
        return false;
    }
}
