/*
 *     QueryImport.java  Aug 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class QueryImport {

    public static boolean hasImported(JavaImport jimport, JavaClassList classes) {
        Iterator it = classes.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (hasImported(jimport, jc)) {
                return true;
            }
        }
        return false;
    }

    public static boolean hasImported(JavaImport jimport, ArrayList nodes) {
        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            JavaImport ji = new JavaImport((Element)it.next());

            if (hasImported(jimport, ji.getName())) {
                return true;
            }
        }
        return false;
    }

    public static boolean hasImported(JavaImport jimport, JavaClass jclass) {
        return hasImported(jimport, jclass.getFqn());
    }

    public static boolean hasImported(JavaImport jimport, String fqn) {
        String qname = jimport.getName();
        int len = qname.indexOf("*");
        
        if (len == -1) {
            if (qname.equals(fqn)) {
                return true;
            }

        } else {
            qname = qname.substring(0, len - 1);
            int len2 = fqn.lastIndexOf(".");

            if (len2 != -1) {
                fqn = fqn.substring(0, len2);
                if (qname.equals(fqn)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean needsImport(JavaClass dclass, JavaClass jc) {
        if (jc.getFqn().startsWith("java.lang")) {
            return false;
        }
        
        if (!QueryClass.isAccessible(dclass, jc)) {
            return false;
        }

        if (QueryClass.hasImported(dclass, jc)) {
            return false;
        }

        if (dclass.isInTheSamePackage(jc) && !jc.isInnerClass()) {
            return false;
        }

        Iterator it = jc.getAllOuterClasses().iterator();
        while (it.hasNext()) {
            JavaClass c = (JavaClass)it.next();
            if (dclass.equals(c)) {
                return false;
            }
        }

        return true;
    }
}
