/*
 *     QueryProject.java  Feb 13, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.query;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.j2xml.JavaXMLText;
import org.jtool.platform.j2xml.ClassXMLText;
import java.util.*;
import org.w3c.dom.*;

public class QueryProject {

    public static ArrayList getAllPackages() {
        ArrayList names = new ArrayList();

        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            Element elem = text.getDocument().getDocumentElement();

            JavaFile jf = new JavaFile(elem);
            if (!names.contains(jf.getPackageName())) {
                names.add(jf.getPackageName());
            }
        }
        return names;
    }

    public static ArrayList getAllJavaFiles() {
        ArrayList files = new ArrayList();
        
        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();

            Element elem = text.getDocument().getDocumentElement();
            JavaFile jf = new JavaFile(elem);
            if (jf != null) {
                files.add(jf);
            }
        }
        return files;
    }

    public static ArrayList getAllJavaFilesInPackage(String name) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFiles().iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (jf.getPackageName().equals(name)) {
                files.add(jf);
            }
        }
        return files;
    }     
 
    public static JavaClassList getAllJavaClasses() {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            Element elem = text.getDocument().getDocumentElement();

             JavaFile jf = new JavaFile(elem);
             classes.addAll(QueryFile.getAllJavaClasses(jf));
        }

        it = Repository.getClassTexts().iterator();
        while (it.hasNext()) {
            ClassXMLText text = (ClassXMLText)it.next();
            Element elem = text.getDocument().getDocumentElement();

            JavaFile jf = new JavaFile(elem);
            classes.addAll(QueryFile.getAllJavaClasses(jf));
        }
        return classes;
    }
   
    public static JavaClassList getAllJavaClassesInProject() {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            Element elem = text.getDocument().getDocumentElement();

             JavaFile jf = new JavaFile(elem);
             classes.addAll(QueryFile.getAllJavaClasses(jf));
        }
        return classes;
    }
   
    public static JavaClassList getAllJavaClassesInPackage(String name) {
        JavaClassList classes = new JavaClassList();

        Iterator it = getAllJavaFilesInPackage(name).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            classes.addAll(jf.getAllClasses());
        }
        return classes;
    }     

    public static ArrayList getAllJavaFilesUsingTheClass(JavaClass jclass) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFiles().iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (isTarget(jf, jclass.getJavaFile()) &&
                QueryFile.getJavaTypesUsingTheClass(jf, jclass).size() != 0) {
                files.add(jf);
            }
        }
        return files;
    }
    
    public static ArrayList getAllJavaFilesUsingTheClassExceptSelf(JavaClass jclass) {
        ArrayList files = new ArrayList();
        
        Iterator it = getAllJavaFilesUsingTheClass(jclass).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (!jclass.isInTheSameFile(jf)) {
                files.add(jf);
            }
        }
        return files;
    }

    public static ArrayList getAllJavaFilesCallingTheMethod(JavaMethod jmethod) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFiles().iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (isTarget(jf, jmethod.getJavaClass().getJavaFile()) &&
                QueryFile.getJavaCallsUsingTheMethod(jf, jmethod).size() != 0) {
                files.add(jf);
            }
        }
        return files;
    }

    public static ArrayList getAllJavaFilesCallingTheMethodExceptSelf(JavaMethod jmethod) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFilesCallingTheMethod(jmethod).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (!jmethod.isInTheSameFile(jf)) {
                files.add(jf);
            }
        }
        return files;
    }

    public static ArrayList getAllJavaFilesUsingTheField(JavaField jfield) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFiles().iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            if (isTarget(jf, jfield.getJavaClass().getJavaFile()) &&
                QueryFile.getJavaVariablesUsingTheField(jf, jfield).size() != 0) {
                files.add(jf);
            }
        }

        return files;
    }

    public static ArrayList getAllFilesUsingTheFieldExceptSelf(JavaField jfield) {
        ArrayList files = new ArrayList();

        Iterator it = getAllJavaFilesUsingTheField(jfield).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();
            
            if (!jfield.isInTheSameFile(jf)) {
                files.add(jf);
            }
        }
        return files;
    }

    private static boolean isTarget(JavaFile jf, JavaFile target) {
        if (target.getPackageName().length() != 0 || target.getDirName().equals(jf.getDirName())) {
            return true;
        }
        return false;
    }
    
    protected static JavaVariableList getJavaVariablesUsingTheField(JavaElement jelem, JavaField jfield) {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = jelem.getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaExpr jr = new JavaExpr(elem);
            if (jr.isVarRef()) {
                JavaVariable jv = new JavaVariable(elem);

                if (!jv.isDeclarationVariable()) {
                    JavaField jf = jv.getUsedJavaField();
                    if (jf != null && jf.equals(jfield)) {
                        vars.add(jv);
                    }
                }
            }
        }
        return vars;
    }
           
    protected static ArrayList getJavaCallsUsingTheMethod(JavaElement jelem, JavaMethod jmethod) {
        ArrayList calls = new ArrayList();

        Iterator it = jelem.getExpressionNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaExpr jr = new JavaExpr(elem);
            if (jr.isMethodCall()) {
                JavaCall ja = new JavaCall(elem);
            
                JavaMethod jm = ja.getCalledJavaMethod();
                if (jm != null && jm.equals(jmethod)) {
                    calls.add(ja);
                }
            }
        }
        return calls;
    }

    public static JavaClassList getJavaClassesUsingTheField(JavaField jfield) {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = QueryProject.getAllJavaFilesUsingTheField(jfield).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            Iterator itc = jf.getAllClasses().iterator();
            while (itc.hasNext()) {
                JavaClass jc = (JavaClass)itc.next();
                
                if (QueryClass.usesTheField(jc, jfield)) {
                    classes.add(jc);
                }
            }
        }
        return classes;
    }

    public static JavaClassList getJavaClassesCallingTheMethod(JavaMethod jmethod) {
        JavaClassList classes = new JavaClassList();
        
        Iterator it = QueryProject.getAllJavaFilesCallingTheMethod(jmethod).iterator();
        while (it.hasNext()) {
            JavaFile jf = (JavaFile)it.next();

            Iterator itc = jf.getAllClasses().iterator();
            while (itc.hasNext()) {
                JavaClass jc = (JavaClass)itc.next();
                
                if (QueryClass.callsTheMethod(jc, jmethod)) {
                    classes.add(jc);
                }
            }
        }
        return classes;
    }

    public static JavaClassList getAnonymousJavaClasses(JavaClass jclass) {
        JavaClassList classes = new JavaClassList();

        Iterator it = getAllJavaClassesInProject().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            if (jc.isAnonymous() && jclass.getFqn().equals(jc.getFqnOfAnonymous())) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public static JavaClass getJavaClassInProject(String fqn) {
        Iterator it = Repository.getJavaTexts().iterator();
        while (it.hasNext()) {
            JavaXMLText text = (JavaXMLText)it.next();
            Element elem = text.getDocument().getDocumentElement();

            JavaFile jf = new JavaFile(elem);
            JavaClass jc = jf.getJavaClass(fqn);
            if (jc != null) {
                return jc;
            }
        }
        return null;
    }
}
