/*
 *     SwitchToPolymorphism.java  Aug 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.statements;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.j2xml.Repository;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.JXConstants;
import org.w3c.dom.*;
import java.util.*;
import javax.swing.JFrame;

public class SwitchToPolymorphism extends StatementRefactoring {
    private ArrayList dclasses;  // DstJavaClass
    private SwitchToPolymorphismTransformer transformer;
    private CFG cfg;

    public SwitchToPolymorphism() {
        super();
    }

    public void setUpWithOptions(JFrame f, HashMap map) {
        frame = f;

        dclasses = new ArrayList();
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            String label = (String)it.next();
            String name = (String)map.get(label);

            JavaStatement jst = new JavaStatement(elem);
            JavaClass jc = Repository.getJavaClass(name, jst.getJavaFile());
            DstJavaClass dclass = new DstJavaClass(label, name, jc);
            dclasses.add(dclass);
        }
    }

    public void preconditions() throws RefactoringException {
        try {
            cfg = new CFG(jstatement.getJavaMethod().getElem());
        } catch (CFGException e) {
            throw new RefactoringException(e.getMessage());
        }

        checkOtherStatementsExceptTheSwitch();

        if (dclasses == null) {
            JavaClassList subclasses = jclass.getDescendants();
            HashMap classMap = createClassMap(subclasses);
            ArrayList switchLabels = getSwitchLabes(jstatement);
            ArrayList candidates = getCandidates(subclasses, switchLabels);

            CorrespondDialog dialog = new CorrespondDialog(frame, "Correspondings", switchLabels, candidates);
            HashMap map = dialog.getCorrespondence();
            if (map.isEmpty()) {
                throw new RefactoringException("");
            }

            dclasses = collectCorrespondingClasses(map, classMap);
        }

        checkSubclasses();
    }

    public void transform() throws RefactoringException {
        transformer = new SwitchToPolymorphismTransformer(jstatement, cfg, dclasses);
        transformer.changeSrc();
        unparse(jfile);

        Iterator it = dclasses.iterator();
        while (it.hasNext()) {
            DstJavaClass dclass = (DstJavaClass)it.next();

            JavaClass jc = dclass.getJavaClass();
            if (jc != null) {
                JavaFile dfile = jc.getJavaFile();
                transformer.changeDst(jc, dclass.getLabel());
                unparse(dfile);
            }
        }
    }

    public String getLog() {
        return "switch statement in " + jmethod.getName() + " of " + jclass.getName();
    }

    private void checkOtherStatementsExceptTheSwitch() throws RefactoringException {
        Node node = jmethod.getFirstChildNode(jmethod.getElem(), JXConstants.StatementElem);  // skip BLOCK
        ArrayList nodes = jmethod.getChildNodes(node, JXConstants.StatementElem);
        if (nodes.size() == 1 && nodes.get(0).equals(jstatement.getElem())) {
            throw new RefactoringException("Not change:" +
              " the method " + jmethod.getName() + " contains statements other than the target switch statement");
        }
    }

    private void checkSubclasses() throws RefactoringException {
        Iterator it = dclasses.iterator();
        while (it.hasNext()) {
            DstJavaClass dclass = (DstJavaClass)it.next();

            JavaClass jc = dclass.getJavaClass();
            if (jc != null && QueryClass.hasTheMethod(jc, jmethod)) {
                throw new RefactoringException("Not change: " +
                  " the method " + jmethod.getName() + " already exists in the source class " + jc.getName());
            }
        }

        if (QueryMethod.callsPrivateMethodsInTheClassExceptSelf(jmethod, jclass)) {
            throw new RefactoringException("Not change:" +
              " the method " + jmethod.getName() + " calls other private methods in the source class " + jclass.getName());
        }
        
        if (QueryMethod.usesPrivateFieldsInTheClass(jmethod, jclass)) {
            throw new RefactoringException("not change:" +
              " the method " + jmethod.getName() + " uses private fields in the source class " + jclass.getName());
        }
    }

    private HashMap createClassMap(JavaClassList subclasses) {
        HashMap map = new HashMap();
        Iterator it = subclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            map.put(jc.getFqn(), jc);
        }
        return map;
    }

    private ArrayList getSwitchLabes(JavaStatement jst) {
        ArrayList labels = new ArrayList();

        NodeList nl = jst.getElem().getElementsByTagName(JXConstants.LabelElem);
        for (int i = 0; i < nl.getLength(); i++) {
            JavaStatement js = new JavaStatement(nl.item(i));

            String label;
            if (js.getKeyword(js.getElem()).equals("case")) {
                Node node = js.getFirstChildNode(js.getElem(), JXConstants.ExpressionElem);
                label = DOM2Code.getCode(node);
            } else {  // js.getKeyword(elem).equals("default")
                label = "default"; 
            }
            labels.add(label);
        }
        return labels;
    }

    private ArrayList getCandidates(JavaClassList subclasses, ArrayList labels) {
        ArrayList candidates = new ArrayList();

        Iterator it = subclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();
            if (!jc.equals(jclass)) {
                candidates.add(jc.getFqn());
            }
        }

        it = labels.iterator();
        while (it.hasNext()) {
            String label = (String)it.next();
            String upperLabel = label.toUpperCase().replace('.', '_');
            while (checkLabel(candidates, upperLabel)) {
                upperLabel = upperLabel.charAt(0) + upperLabel;
            }
            candidates.add(upperLabel);
        }
        return candidates;
    }

    private boolean checkLabel(ArrayList list, String label) {
        Iterator it = list.iterator();
        while (it.hasNext()) {
            String name = (String)it.next();
            if (label.equals(name)) {
                return true;
            }
        }
        return false;
    }

    private ArrayList collectCorrespondingClasses(HashMap map, HashMap classMap) {
        ArrayList classes = new ArrayList(); 

        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            String label = (String)it.next();
            String name = (String)map.get(label);
            JavaClass jc = (JavaClass)classMap.get(name);

            DstJavaClass dclass;
            if (jc != null) {
                dclass = new DstJavaClass(label, jc.getFqn(), jc);
            } else {
                dclass = new DstJavaClass(label, name, null);
            }
            classes.add(dclass);
        }
        return classes;
    }
}

class DstJavaClass {
    private String label;
    private String name;
    private JavaClass jclass;

    DstJavaClass(String label, String name, JavaClass jclass) {
        this.label = label;
        this.name = name;
        this.jclass = jclass;
    }

    String getLabel() {
        return label;
    }

    String getName() {
        return name;
    }

    JavaClass getJavaClass() {
        return jclass;
    }
}    
