/*
 *     CodeSelection.java  Aug 16, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class CodeSelection {
    private Document document;
    private int startPos, endPos;
    private ArrayList foundIds = new ArrayList();  // String
    private ArrayList foundElems = new ArrayList();  // Element

    public CodeSelection(Document doc, int start) {
        this(doc, start, start);
    }

    public CodeSelection(Document doc, int start, int end) {
        document = doc;
        startPos = start;
        endPos = end;
    }

    public void find() {
        try {
            checkNode(document.getDocumentElement());

        } catch (Exception e) {
            foundIds.clear();
            foundElems.clear();
        }
    }

    public ArrayList getElems() {
        return foundElems;
    }

    public Element getRepresentativeElem() {
        if (foundIds.size() > 0) {
            return GetElem.getElem(document, (String)foundIds.get(0));
        }
        return null;
    }

    public ArrayList getIds() {
        return foundIds;
    }

    private int currentPos = 0;
    private HashMap elems = new HashMap();

    private void checkNode(Node node) {
        if (node.getNodeType() == Node.TEXT_NODE) {
            String token = node.getNodeValue();

            if (isWithin(currentPos, currentPos + token.length() - 1)) {
                Element elem = (Element)node.getParentNode();
                String defid = null;

                if (elem.getNodeName().equals(JXConstants.IdentElem)) {
                    Element pelem = (Element)elem.getParentNode();
                    defid = elem.getAttribute(JXConstants.DefIdAttr);

                    if (pelem.getNodeName().equals(JXConstants.ExpressionElem)) {
                        JavaExpr jexpr = new JavaExpr(pelem);
                        if (jexpr.isMethodCall() || jexpr.isVarRef()) {
                            defid = elem.getAttribute(JXConstants.DefIdAttr);
                        }

                    } else if (pelem.getNodeName().equals(JXConstants.TypeElem)) {
                        Element ppelem = (Element)pelem.getParentNode();
                        if (ppelem.getNodeName().equals(JXConstants.ExpressionElem)) {
                            JavaExpr jexpr = new JavaExpr(ppelem);
                            defid = ppelem.getAttribute(JXConstants.IdAttr);
                        }
                    }

                } else if (elem.getNodeName().equals(JXConstants.KeywordElem)) {
                    Element pelem = (Element)elem.getParentNode();
                    defid = "";

                    if (pelem.getNodeName().equals(JXConstants.StatementElem) ||
                        pelem.getNodeName().equals(JXConstants.ExpressionElem)) {
                        defid = pelem.getAttribute(JXConstants.IdAttr);
                    }

                } else if (elem.getNodeName().equals(JXConstants.OperatorElem)) {
                    defid = "";

                } else if (elem.getNodeName().equals(JXConstants.LiteralElem)) {
                    defid = "";

                } else if (elem.getNodeName().equals(JXConstants.CommentElem)) {
                    defid = elem.getAttribute(JXConstants.IdAttr);

                } else if (elem.getNodeName().equals(JXConstants.SpaceElem)) {
                    defid = elem.getAttribute(JXConstants.IdAttr);

                } else if (elem.getNodeName().equals(JXConstants.NewLineElem)) {
                    defid = elem.getAttribute(JXConstants.IdAttr);
                }

                Element lelem = (Element)getLabelNode(elem);
                if (lelem != null &&
                    (elem.getNodeName().equals(JXConstants.IdentElem) ||
                     elem.getNodeName().equals(JXConstants.LiteralElem))) {
                    defid = lelem.getAttribute(JXConstants.IdAttr);
                }

                if (defid != null) {
                    foundElems.add(elem);
                    if (!defid.equals("")) {
                        foundIds.add(defid);
                    }
                    //System.out.println("BINGO: " + token + " " + defid + " " + elem);
                }
            }
            currentPos = currentPos + token.length();

        } else if (node.getNodeType() == Node.ELEMENT_NODE) {
            Element elem = (Element)node;
        
            String sort = node.getNodeName();
            if (sort.equals(JXConstants.PackageElem) ||
                sort.equals(JXConstants.ImportElem) ||
                sort.equals(JXConstants.ClassElem) ||
                sort.equals(JXConstants.InterfaceElem) ||
                sort.equals(JXConstants.MethodElem) ||
                sort.equals(JXConstants.ConstructorElem) ||
                sort.equals(JXConstants.FieldElem) ||
                sort.equals(JXConstants.ParameterElem) ||
                sort.equals(JXConstants.LocalElem) ||
                sort.equals(JXConstants.StatementElem)) {

                if (sort.equals(JXConstants.StatementElem)) {
                    sort = elem.getAttribute(JXConstants.SortAttr);
                }
                
                String id = elem.getAttribute(JXConstants.IdAttr);
                if (id.length() != 0 && sort.length() != 0) {
                    elems.put(id, sort);
                }
            }

            NodeList nl = node.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                Node child = nl.item(i);
                checkNode(child);
            }
        }
    }

    private boolean isWithin(int begin, int end) {
        for (int pos = begin; pos <= end; pos++) {
            if (startPos <= pos && pos <= endPos) {
                return true;
            }
        }
        return false;
    }

    private Node getLabelNode(Node node) {
        while (node != null &&
               !node.getNodeName().equals(JXConstants.LabelElem) &&
               !node.getNodeName().equals(JXConstants.FileElem)) {
            node = node.getParentNode();
        }

        if (node.getNodeName().equals(JXConstants.LabelElem)) {
            return node;
        }
        return null;
    }
}
