/*
 *     Dom2Code.java  Feb 20, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.platform.xml.dom.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;
import java.awt.Color;

public class Dom2Code {
    private static StringBuffer buf;
    private static int pos;
    private static int begin;
    private static ArrayList highlighters;  // HighlightToken
    private static String sensitiveAction;
    private static String ignoredAction;
    private static int referredIdent;
    private static int sensitiveNum;
    private static int ignoredNum;

    private static void init() {
        buf = new StringBuffer();

        pos = 0;
        begin = 0;
        referredIdent = -1;
        sensitiveNum = 0;
        ignoredNum = 0;

        highlighters = new ArrayList();
    }

    public static String getOldCode(Document doc) {
        init();
        sensitiveAction = RefactoringConstants.DELETED;
        ignoredAction = RefactoringConstants.INSERTED;

        writeNode(doc.getDocumentElement());
        return buf.toString();
    }

    public static String getNewCode(Document doc) {
        init();
        sensitiveAction = RefactoringConstants.INSERTED;
        ignoredAction = RefactoringConstants.DELETED;

        writeNode(doc.getDocumentElement());
        return buf.toString();
    }

    private static void writeNode(Node node) {
        if (node.getNodeType() == Node.TEXT_NODE) {
            
            if (ignoredNum == 0) {
                String token = node.getNodeValue();
                buf.append(token);
                pos = pos + token.length();
            }

        } else if (node.getNodeType() == Node.ELEMENT_NODE) {

            String action = ((Element)node).getAttribute(RefactoringConstants.ChangeAttr);
            if (action.equals(ignoredAction)) {
                ignoredNum++;

            } else if (ignoredNum == 0) {
                if (action.equals(sensitiveAction)) {
                    if (sensitiveNum == 0) {
                        begin = pos;
                    }
                    sensitiveNum++;
                }
                if (action.equals(RefactoringConstants.REFERENCE)) {
                    referredIdent = pos;
                } else {
                    referredIdent = -1;
                }
            }

            for (Node child = node.getFirstChild();
                 child != null; child = child.getNextSibling()) {
                writeNode(child);
            }

            if (action.equals(ignoredAction)) {
                ignoredNum--;

            } else if (ignoredNum == 0) {
                if (action.equals(sensitiveAction)) {
                    setHighlight(begin, pos, sensitiveAction);
                    sensitiveNum--;
                }
                if (referredIdent != -1) {
                    setHighlight(referredIdent, pos, RefactoringConstants.REFERENCE);
                }
            }
        }
    }

    private static void setHighlight(int begin, int end, String highlight) {
        while (begin < end - 1 && (buf.charAt(begin) == '\n' || buf.charAt(begin) == ' ')) {
            begin++;
        }
        while (begin < end - 1 && (buf.charAt(end - 1) == '\n' || buf.charAt(end - 1) == ' ')) {
            end--;
        }

        if (begin == end) {
            return;
        }

        setHighlightWithColor(begin, end, highlight);
    }        

    private static void setHighlightWithColor(int begin, int end, String highlight) {
        Color color = null;

        if (highlight.equals(RefactoringConstants.DELETED)) {
            color = Color.yellow;

        } if (highlight.equals(RefactoringConstants.INSERTED)) {
            color = Color.green;

        } if (highlight.equals(RefactoringConstants.REFERENCE)) {
            color = Color.cyan;
        } 

        if (color != null) {
            HighlightToken ht = new HighlightToken(begin, end, color);
            highlighters.add(ht);
        }
    }

    public static ArrayList getHighlights() {
        return highlighters;
    }
}
