/*
 *     FieldCode.java  Dec 3, 2003
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class FieldCode {
    private Modifier modifier = new Modifier();
    private String type;
    private String name;

    private FieldCode() {
    }

    public FieldCode(String type, String name) {
        this.type = type;
        this.name = name;
    }

    public FieldCode(JavaField jf) {
        this(jf.getType(), jf.getName());

        if (jf.isPublic()) {
            addModifier(Modifier.PUBLIC);

        } else if (jf.isProtected()) {
            addModifier(Modifier.PROTECTED);

        } else if (jf.isPrivate()) {
            addModifier(Modifier.PRIVATE);

        } else if (jf.isStatic()) {
            addModifier(Modifier.STATIC);

        } else if (jf.isFinal()) {
            addModifier(Modifier.FINAL);
        }
    }

    public void addModifier(String keyword) {
        modifier.add(keyword);
    }

    public void addModifier(int index, String keyword) {
        modifier.add(index, keyword);
    }

    public void removeModifier(String keyword) {
        modifier.remove(keyword);
    }

    public String getName() {
        return name;
    }

    public String getType() {
        return type;
    }

    public String getCode() {
        StringBuffer buf = new StringBuffer();
        buf.append(getFieldCode());
        return buf.toString();
    }

    public String getFieldCode() {
        StringBuffer buf = new StringBuffer();

        buf.append(modifier.toString());
        if (type != null) {
            buf.append(" ");
            buf.append(type);
        }

        if (buf.length() != 0) {
            buf.append(" ");
        }

        buf.append(name);
        buf.append(";");

        return buf.toString();
    }
}
