/*
 *     UndoChecker.java  Nov 23, 2005
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.util;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.j2xml.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class UndoChecker {

    public static boolean check(String fileName, String undoInfo) {
        if (undoInfo.length() == 0) {
            return false;
        }

        JavaXMLText text = Repository.getJavaXMLText(fileName);
        if (text != null) {
            JavaFile jfile = text.getJavaFile();

            String info = undoInfo.substring(2);
            int index = info.indexOf("#");
            String prefix = info.substring(0, index);
            String suffix = info.substring(index + 1);

            if (undoInfo.startsWith("C:")) {
                return checkClass(jfile, suffix);

            } else if (undoInfo.startsWith("M:")) {
                return checkMethod(jfile, prefix, suffix);

            } else if (undoInfo.startsWith("F:")) {
                return checkField(jfile, prefix, suffix);
            }
        }
        return false;
    }

    private static boolean checkClass(JavaFile jfile, String cname) {
        JavaClass jc = QueryFile.getJavaClassWithTheSameName(jfile, cname);
        return jc != null;
    }

    private static boolean checkMethod(JavaFile jfile, String cname, String mname) {
        Iterator it = jfile.getAllClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (jc.getName().equals(cname)) {
                return checkMethod(jc.getAncestors(), mname) && checkMethod(jc.getDescendants(), mname);
            }
        }
        return false;
    }

    private static boolean checkMethod(JavaClassList jclasses, String mname) {
        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            JavaMethod jm = jc.getJavaMethod(mname);
            if (jm != null) {
                return true;
            }
        }
        return false;
    }

    private static boolean checkField(JavaFile jfile, String cname, String fname) {
        Iterator it = jfile.getAllClasses().iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            if (jc.getName().equals(cname)) {
                return checkField(jc.getAncestors(), fname) && checkField(jc.getDescendants(), fname);
            }
        }
        return false;
    }

    private static boolean checkField(JavaClassList jclasses, String fname) {
        Iterator it = jclasses.iterator();
        while (it.hasNext()) {
            JavaClass jc = (JavaClass)it.next();

            JavaField jf = jc.getJavaField(fname);
            if (jf != null) {
                return true;
            }
        }
        return false;
    }
}
