/*
 *     RenameVariable.java  Aug 15, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.variables;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import javax.swing.JFrame;

public class RenameVariable extends VariableRefactoring {
    private String newName = null;
    private RenameVariableTransformer transformer;

    public RenameVariable() {
        super();
    }

    public void setUpWithOptions(JFrame f, String newName) {
        frame = f;
        this.newName = newName;
    }

    public void preconditions() throws RefactoringException {
        if (newName == null) {
            RenameDialogNoReference dialog = new RenameDialogNoReference(frame, "Rename Variable", jvar.getName());
            newName = dialog.getNewName();

            if (newName == null) {
                throw new RefactoringException("");
            }
        }

        checkAlreadyUsed();
    }

    public void transform() throws RefactoringException {
        transformer = new RenameVariableTransformer(jvar, newName);
        transformer.changeSrc();
        unparse(jfile);
    }

    public String getLog() {
        return "from " + jvar.getName() + " to " + newName;
    }

    private void checkAlreadyUsed() throws RefactoringException {
        if (QueryMethod.hasTheVariable(jmethod, newName)) {
            throw new RefactoringException("Not rename:" +
              " the variable " + newName + " already exists in method " + jmethod.getName());
        }

        if (QueryClass.hasTheField(jclass, newName)) {
            throw new RefactoringException("Not rename:" +
              " the field " + newName + " alreasy exists in class " + jclass.getName());
        }
    }
}
