/*
 *     SplitVariable.java  Aug 17, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.variables;
import org.jtool.jrbx.refactor.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.jrbx.refactor.dialog.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.graph.pdg.*;
import java.util.*;
import org.w3c.dom.*;
import javax.swing.JFrame;

public class SplitVariable extends VariableRefactoring {
    private String newName = null;
    private JavaVariable jv;
    private SplitVariableTransformer transformer;
    private CFG cfg;
    private PDG pdg;
    private GraphCompSet nodesToBeChanged = new GraphCompSet();

    public SplitVariable() {
        super();
    }

    public void setUpWithOptions(JFrame f, String newName) {
        frame = f;
        this.newName = newName;
    }

    public void preconditions() throws RefactoringException {
        try {
            cfg = new CFG(jvar.getJavaMethod().getElem());
            pdg = new PDG(cfg);
        } catch (CFGException e) {
            throw new RefactoringException(e.getMessage());
        }

        canSplit();

        if (newName == null) {
            RenameDialogNoReference dialog = new RenameDialogNoReference(frame, "Split Variable", jvar.getName());
            newName = dialog.getNewName();

            if (newName == null) {
                throw new RefactoringException("");
            }
        }

        checkAlreadyUsed();
    }

    public void transform() throws RefactoringException {
        transformer = new SplitVariableTransformer(jv, cfg, nodesToBeChanged, newName);
        transformer.changeSrc();
        unparse(jfile);
    }

    public String getLog() {
        return "into" + jvar.getName() + " and " + newName;
    }

    private void canSplit() throws RefactoringException {
        Element selectedElem = (Element)selectedElems.get(0);
        jv = new JavaVariable(selectedElem.getParentNode());

        CFGNode node = getCFGNode(jv);
        if (node == null) {
            throw new RefactoringException("Invalid refactoring");
        }
        collectNodesToBeChanged(pdg.getNode(node));
        if (!possibleToSplit(nodesToBeChanged)) {
            throw new RefactoringException("Not split:" +
              " the variable " + newName + " is imposible to split");
        }
    }

    private void checkAlreadyUsed() throws RefactoringException {
        if (QueryMethod.hasTheVariable(jmethod, newName)) {
            throw new RefactoringException("Not split:" +
              " the variable " + newName + " already exists in method " + jmethod.getName());
        }

        if (QueryClass.hasTheField(jclass, newName)) {
            throw new RefactoringException("Not split:" +
              " the field " + newName + " alreasy exists in class " + jclass.getName());
        }
    }

    private GraphCompSet nodes = new GraphCompSet();

    private void collectNodesToBeChanged(PDGNode node) {
        if (node != null) {
            collectNodesToBeChanged(node, nodesToBeChanged);
        }
    }

    private CFGNode getCFGNode(JavaVariable var) {
        Element elem = (Element)var.getStatementNode();

        Iterator it = cfg.getNodes().iterator();
        while (it.hasNext()) {
            CFGNode n = (CFGNode)it.next();

            Element e = n.getElem();
            if (e != null && e.equals(elem)) {
                return n;
            }
        }
        return null;
    }

    private void collectNodesToBeChanged(PDGNode node, GraphCompSet prevnodes) {
        nodes.clear();
        QueryGraph.traverseBackwardDD(nodes, node, jv);

        GraphCompSet tmpNodes = new GraphCompSet(nodes);
        Iterator it = tmpNodes.iterator();
        while (it.hasNext()) {
            PDGNode n = (PDGNode)it.next();
            QueryGraph.traverseForwardDD(nodes, n, jv);
        }
        nodesToBeChanged = nodesToBeChanged.union(nodes);

        GraphCompSet diff = nodesToBeChanged.difference(prevnodes);
        it = diff.iterator();
        while (it.hasNext()) {
            PDGNode n = (PDGNode)it.next();
            collectNodesToBeChanged(n, nodesToBeChanged);
        }
    }

    private boolean possibleToSplit(GraphCompSet cnodes) {
        Iterator it = pdg.getNodes().difference(cnodes).iterator();
        while (it.hasNext()) {
            PDGNode node = (PDGNode)it.next();

            if (QueryGraph.isDefinedAt(node.getCFGNode(), jv)) {
                return true;
            }
        }
        return false;
    }

    public static boolean isDefinedAt(CFGNode node, JavaVariable jv) {
        if (node.hasDefVariable()) {
            CFGStatement n = (CFGStatement)node;
            if (n.containsDefVariable(jv)) {
                return true;
            }
        }
        return false;
    }
}
