/*
 *     SplitVariableTransformer.java  Aug 17, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.jrbx.refactor.variables;
import org.jtool.jrbx.refactor.util.*;
import org.jtool.jrbx.refactor.query.*;
import org.jtool.platform.xml.wrapper.*;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.graph.pdg.*;
import java.util.*;
import org.w3c.dom.*;

public class SplitVariableTransformer extends RefactoringTransformer {
    private JavaVariable jvar;
    private String newName;
    private CFG cfg;
    private PDG pdg;
    private GraphCompSet pdgNodesToBeChanged;
    private GraphCompSet cfgNodesToBeChanged;

    private SplitVariableTransformer() {
    }

    public SplitVariableTransformer(JavaVariable jvar, CFG cfg, GraphCompSet nodes, String newName) {
        this.jvar = jvar;
        this.newName = newName;
        this.cfg = cfg;
        this.pdg = pdg;
        pdgNodesToBeChanged = nodes;

        cfgNodesToBeChanged = new GraphCompSet();
        Iterator it = pdgNodesToBeChanged.iterator();
        while (it.hasNext()) {
            CFGNode cfgNode = (CFGNode)it.next();
            cfgNodesToBeChanged.add(cfgNode);
        }
    }

    public void changeSrc() {
        CFGNode declNode = QueryGraph.getDeclaraionCFGNode(cfg, cfgNodesToBeChanged, jvar);
        if (declNode != null) {
            changeVariableName(cfgNodesToBeChanged, declNode);
        }
    }

    private void changeVariableName(GraphCompSet cfgNodes, CFGNode declNode) {
        Iterator it = cfg.getNodes().iterator();
        while (it.hasNext()) {
            CFGNode cfgNode = (CFGNode)it.next();

            boolean isContained = cfgNodes.contains(cfgNode);
            boolean needsType = insertDeclaration(cfgNode, declNode, isContained);

            if (isContained) {
                JavaStatement jst = new JavaStatement(cfgNode.getElem());
                if (needsType) {
                    String type = jvar.getType();
                    changeUsingName(jst.getDefVariables(), type + " " + newName);
                } else {
                    changeUsingName(jst.getDefVariables(), newName);
                }

                changeUsingName(jst.getUseVariables(), newName);
            }
        }
    }

    private boolean insertDeclaration(CFGNode node, CFGNode decl, boolean isContained) {
        if (node.equals(decl)) {
            if (isContained) {
            
                if (QueryGraph.isDefinedAt(decl, jvar)) {
                    if (!decl.isLocalSt()) {
                        return true;
                    }
                    
                } else {
                    insertDeclaration(decl);
                }
                
            } else {
                insertDeclaration(decl);
            }
        }
        return false;
    }

    private void insertDeclaration(CFGNode node) {
        String space = getSpace(node.getElem());
        String declStr = jvar.getType() + " " + newName + ";\n" + space;
        insertCodeBefore(declStr, node.getElem());
    }

    private void changeUsingName(JavaVariableList vars, String name) {
        Iterator it = vars.iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();

            if ((jv.isLocal() || jv.isParameter()) &&
                jv.getName().equals(jvar.getName())) {
                rename(name, jv);
            }
        }
    }
}
